"""Daschat base schemas."""

from datetime import datetime
from enum import Enum
from pathlib import Path
from typing import Any, Dict, List, Optional

from pydantic import AnyHttpUrl, BaseModel, Extra, Field, stricturl

from .types import MessageDispatchEnum, MessageTypeEnum


class Config(BaseModel):
    class Config:
        validate_assignment = True

    label: str = Field(
        ...,
        max_length=255,
        title="Label",
        description="This config label.",
        example="Some config label",
    )
    key: str = Field(
        ...,
        max_length=50,
        title="Config key",
        description="Config unique key.",
        example="unique-configkey-ckasnzjot0001p2xutk68nys8",
    )
    value: Dict[str, Any] = Field(
        ..., title="Config value", description="Config value in json format."
    )
    status: int = Field(
        1, title="Config status", description="Config status.", example="1", gt=-1, lt=2
    )
    id: str = Field(
        ...,
        max_length=25,
        title="SID",
        description="The ID of the config.",
        example="ckasokq6g0000yvxuigfa2agy",
    )
    owner_id: str = Field(
        ...,
        max_length=25,
        title="Owner ID",
        description="The ID of the owner of this config.",
        example="ckasolg5o0001yvxuzdty0jrd",
    )


class StateItemSchema(BaseModel):
    class Config:
        validate_assignment = True

    id: str = Field(
        ...,
        max_length=25,
        title="State ID",
        description="State ID value.",
        example="optin",
    )

    data: Dict[str, Any] = Field(
        {}, title="State Data", description="State saved data."
    )
    order: int = Field(
        0,
        title="Order",
        description="Order in wich the plugin was executed.",
        example=0,
        gt=-1,
        lt=100,
    )
    state: str = Field(
        "init",
        max_length=25,
        title="State",
        description="Current state value.",
        example="init",
    )


class StatesSchema(BaseModel):
    class Config:
        validate_assignment = True

    channel_in: Optional[List[StateItemSchema]] = []


class HandsoffDataSchema(BaseModel):
    class Config:
        validate_assignment = True

    namespace: str = Field(
        ...,
        max_length=255,
        title="Namespace",
        description="Which plugin this data belongs to.",
        example="rocketchat",
    )
    key: str = Field(
        ...,
        max_length=50,
        title="Data key",
        description="Data unique key.",
        example="chat",
    )
    value: Dict[str, Any] = Field(
        ..., title="Data value", description="Data value in json format."
    )


class Chat(BaseModel):
    class Config:
        validate_assignment = True

    id: str = Field(
        ...,
        max_length=25,
        title="SID",
        description="The ID of this chat.",
        example="ckasokq6g0000yvxuigfa2000",
    )
    owner_id: str = Field(
        ...,
        max_length=25,
        title="Owner ID",
        description="The ID of the owner of this chat.",
        example="ckasolg5o0001yvxuzdty0222",
    )
    contact_id: str = Field(
        ...,
        max_length=25,
        title="Contact ID",
        description="The contact ID this chat call.",
        example="ckasolg5o0001yvxuzdty0111",
    )
    created_at: datetime
    channel: str = Field(
        ...,
        max_length=25,
        title="Channel",
        description="Channel used by this chat.",
        example="whatsapp",
    )
    channel_plugin: str = Field(
        ...,
        max_length=25,
        title="Channel Plugin",
        description="Channel plugin used by this chat.",
        example="twwhatsapp",
    )
    channel_uid: str = Field(
        ...,
        max_length=255,
        title="Channel UID",
        description="Contact from UID.",
        example="5527999000000",
    )
    channel_contact_uid: str = Field(
        ...,
        max_length=255,
        title="Channel Contact UID",
        description="Channel contact UID.",
        example="5527999000011",
    )
    channel_config: str = Field(
        ...,
        max_length=50,
        title="Channel Config",
        description="Channel config key.",
        example="twwhatsapp-ckasnzjot0001p2xutk68n000",
    )
    handson: str = Field(
        ...,
        max_length=25,
        title="Handson",
        description="Channel handson PLUGIN.",
        example="tiledesk",
    )
    handson_config: str = Field(
        ...,
        max_length=50,
        title="Handson Config Key",
        description="Channel handson config key.",
        example="tiledesk-ckbl7w4nq0000zuxuiibic000",
    )
    subject: Dict[str, Any] = Field(..., title="Subject", description="Chat subject.")
    status: int = Field(
        ..., title="Status", description="Chat status.", example="1", gt=-1, lt=1025
    )
    # state: Dict[str, Any] = Field({}, title="State", description="Chat state machine.")
    state: Optional[StatesSchema]
    handsoff_cid: Optional[str] = Field(
        None,
        max_length=128,
        title="Handsoff chat ID",
        description="Handsoff app own chat ID",
        example="BxdYnAjpu3PTciw6Z",
    )
    handsoff_data: Optional[List[HandsoffDataSchema]] = Field(
        [],
        title="Handsoff app data",
        description="handsoff data received from the plugin and that need to be stored.",
    )


class Contact(BaseModel):
    class Config:
        validate_assignment = True

    id: str = Field(
        ...,
        max_length=25,
        title="SID",
        description="The ID of the Contact call.",
        example="ckasokq6g0000yvxuigfa2agy",
    )
    owner_id: str = Field(
        ...,
        max_length=25,
        title="Owner ID",
        description="The ID of the owner of this Contact call.",
        example="ckasolg5o0001yvxuzdty0jrd",
    )
    created_at: datetime
    full_name: str = Field(
        ...,
        max_length=255,
        title="Full Name",
        description="Contact full name.",
        example="John Doe",
    )
    email: str = Field(
        ...,
        max_length=255,
        title="Email",
        description="Contact email.",
        example="johndoe@domain.com",
    )
    channel: str = Field(
        ...,
        max_length=25,
        title="Channel",
        description="Channel used by this contact.",
        example="whatsapp",
    )
    channel_id: str = Field(
        ...,
        max_length=255,
        title="Channel ID",
        description="Contact channel ID.",
        example="5527999000000",
    )
    profile: Dict[str, Any] = Field(
        ..., title="Profile", description="Contact profile."
    )
    status: int = Field(
        ..., title="Status", description="Contact status.", example="1", gt=-1, lt=1025
    )


class CustomFields(BaseModel):
    class Config:
        validate_assignment = True

    key: str = Field(
        ...,
        max_length=32,
        title="Key",
        description="Unique key.",
        example="channel",
    )
    value: str = Field(
        ...,
        max_length=128,
        title="Value",
        description="Value of the key.",
        example="whatsapp",
    )
    overwrite: bool = Field(
        ...,
        title="Overwrite",
        description="Overwrite existent value.",
    )


class MediaMetaData(BaseModel):
    class Config:
        validate_assignment = True

    name: str = Field(
        ...,
        max_length=128,
        title="File Name",
        description="Original media file name.",
        example="filename.png",
    )
    path: Path = Field(
        ...,
        title="Media Path",
        description="The valid absolute path of the media in the storage system",
        example="/media/filename.png",
    )
    src: AnyHttpUrl = Field(
        ...,
        title="Media URL",
        description="The valid absolute URL of the media. Needs to be HTTPS",
        example="https://mydomain.com/media/filename.png",
    )
    type: str = Field(
        ...,
        max_length=128,
        title="Type",
        description="Media type.",
        example="image/png",
    )
    height: Optional[int] = Field(
        0, title="Height", description="Height for image media.", example=460
    )
    width: Optional[int] = Field(
        0, title="Width", description="Width for image media.", example=817
    )
    uid: Optional[str] = Field(
        "",
        max_length=64,
        title="Media UID",
        description="Media optional UID.",
        example="AAAAAElFTkSuQmCC",
    )


class MessageSchema(BaseModel):
    class Config:
        validate_assignment = True

    dispatch: MessageDispatchEnum = Field(
        MessageDispatchEnum.gateway,
        title="Dispatch Module",
        description="Dispatch message with this module",
        example="gateway",
    )
    message_type: MessageTypeEnum = Field(
        MessageTypeEnum.text, title="Type", description="Message type", example="text"
    )
    content: str = Field(
        ...,
        max_length=2048,
        title="Content",
        description="Message content.",
        example="This is a message content.",
    )
    metadata: Optional[MediaMetaData]
    agent_id: str = Field(
        None,
        max_length=128,
        title="Agent ID",
        description="ID of the agent who wrote the message.",
        example="BxdYnAjpu3PTciw6Z",
    )
    agent_name: str = Field(
        "Bot",
        max_length=128,
        title="Agent Name",
        description="Name of the agent.",
        example="Ruth A.",
    )
    Department: str = Field(
        "General",
        max_length=128,
        title="Department",
        description="Department associated with the message.",
        example="Department",
    )


class HandsoffInfoDataSchema(BaseModel):
    class Config:
        validate_assignment = True

    version: str = Field(
        ...,
        max_length=64,
        title="Version",
        description="Handsoff app version.",
        example="v4.0.1",
    )
    departments: Optional[List[Dict[str, Any]]] = Field(
        [],
        title="Departments",
        description="Handsoff departments.",
    )
    agents: Optional[List[Dict[str, Any]]] = Field(
        [],
        title="Agents",
        description="Handsoff agents.",
    )
    business_hours: Optional[List[Dict[str, Any]]] = Field(
        [],
        title="Business Hours",
        description="Business hours of human attendants",
    )


class HandsoffInfoSchema(BaseModel):
    class Config:
        validate_assignment = True

    success: bool = Field(
        ...,
        title="Success",
        description="Handsoff app is online.",
    )
    status_code: int = Field(
        200,
        title="Status Code",
        description="Request status code.",
        example="200",
        gt=99,
        lt=600,
    )
    data: HandsoffInfoDataSchema = Field(
        ..., title="Data", description="Handsoff app data."
    )


class ChannelConfig(BaseModel):
    class Config:
        validate_assignment = True

    app: str = Field(
        ...,
        max_length=255,
        title="Label",
        description="App ID",
        example="Channel App ID",
    )
    channel_type: str = Field(
        ...,
        max_length=25,
        title="Channel Type",
        description="Channel type",
        example="whatsapp",
    )
    endpoint: AnyHttpUrl = Field(
        ...,
        title="Api Endpoint",
        description="Channel API endpoint. Needs to be HTTPS",
        example="https://api.gupshup.io",
    )
    user: str = Field(
        None,
        max_length=256,
        title="User",
        description="Channel user if needed",
        example="email@domain.com",
    )
    password: str = Field(
        None,
        max_length=64,
        title="Password",
        description="Channel password if needed",
        example="SomeGoodPassword",
    )
    api_key: str = Field(
        None,
        max_length=256,
        title="Api key",
        description="Api key if needed",
        example="ckasolg5o0001yvxuzdty0jrdckasnzjot0001p2xutk68nys8",
    )
    from_uid: str = Field(
        ...,
        max_length=255,
        title="From UID",
        description="Channel sender UID.",
        example="5527999000000",
    )
    extra: Dict[str, Any] = Field(
        {}, title="Extra", description="Additional data for the channel plugin."
    )


class ResultFieldSchema(BaseModel):
    class Config:
        validate_assignment = True
        extra = Extra.forbid

    status: bool = Field(
        True,
        title="Status",
        description="Call result status.",
    )
    id: str = Field(
        ...,
        max_length=32,
        title="Result ID",
        description="Result message ID in Daschat.",
        example="SUCCESS",
    )
    params: dict = Field(
        {},
        title="Params",
        description="Params for the result message.",
        example={"user": "theusername"},
    )


class DispatchCallOutSchema(BaseModel):
    class Config:
        validate_assignment = True
        extra = Extra.forbid

    result: ResultFieldSchema = Field(
        ...,
        title="Result",
        description="Result of the dispatch module call.",
    )
    messages: Optional[List[MessageSchema]] = Field(
        [],
        title="Messages",
        description="Messages from dispatch module to be delivered by Daschat.",
        example="See schema MessageSchema",
    )



""" Daschat handoff metaclasses

This module must be used to create any handoff plugin to be used with the `Daschat
System`.

Todo:
    * Write docs
    * You have to also use ``sphinx.ext.todo`` extension

Some docs about how it is made:

* https://realpython.com/factory-method-python/

* https://medium.com/@geoffreykoh/implementing-the-factory-pattern-via-dynamic-registry-and-python-decorators-479fc1537bbe

"""

import json

# Standard imports
from abc import ABCMeta, abstractmethod
from enum import Enum
from typing import Any, Callable, List, Optional

# Third party imports
import cuid
from loguru import logger
from pydantic import BaseSettings

from .schemas import Chat, Config, Contact, HandsoffInfoSchema, MessageSchema

# Application imports
from .types import ActionToTypeEnum


class ChatIdTypesEnum(int, Enum):
    """Possible ways handsoff app associate chat_id with Daschat

    Args:
        int ([type]): [description]
        Enum ([type]): [description]
    """

    DASCHAT = 0
    OWN = 1


class WebhookTypesEnum(int, Enum):
    """Webhook types processed by this plugin

    https://stackoverflow.com/questions/24487405/enum-getting-value-of-enum-on-string-conversion

    Args:
        int ([type]): [description]
        Enum ([type]): [description]
    """

    UNKNOW = 0
    CHAT_START = 10
    CHAT_QUEUED = 30
    CHAT_SESSION = 40
    CHAT_TAKEN = 50
    CHAT_FORWARDED = 51
    MESSAGE = 60

    # def __repr__(self):
    #     return self.value


# Acceptable medias
MEDIA_TYPES = {
    "audio": [
        "audio/aac",
        "audio/mp4",
        "audio/amr",
        "audio/mpeg",
        "audio/ogg; codecs=opus",
    ],
    "image": ["image/jpeg", "image/png"],
    "sticker": ["image/webp"],
    "video": ["video/mp4", "video/3gpp"],
}


class HandoffBase(metaclass=ABCMeta):
    """Base class for an handoff plugin"""

    request_id: str = cuid.cuid()
    config: Config
    data: dict = {}
    plugin: str = "generic"
    _assignment: str = "handoff"
    api_url: str
    api_path: str
    _service_api: Callable
    init_kwargs: dict
    logged: bool = False
    headers: dict = {"Content-Type": "application/json"}
    _actions: List[dict] = []
    _agent_msg_bom: str = "=========== **[BOT] Mensagem do sistema - Início**"
    _agent_msg_eom: str = "=========== **[BOT] Mensagem do sistema - Fim**"
    _webhook_type: int = WebhookTypesEnum.MESSAGE.value
    status_code: int = 200
    chat_id_type: int = ChatIdTypesEnum.DASCHAT.value
    chat_id: str
    chat_status: int = 0
    request_status: int = 0
    have_msg: bool = True
    response_type = "application/json"
    response_content: str
    output_msg: Optional[List[MessageSchema]] = []
    agent_msg: Optional[List[MessageSchema]] = []
    settings: BaseSettings
    media_bucket: str
    data_in: str
    info_data: HandsoffInfoSchema

    def __init__(self, config: Config, settings: Any, data: dict, **kwargs):
        """Initialize handsoff plugin

        Args:
            config (Config): Config of channel, hansdoff app, plugins, etc
            settings (Any): Settinhs of the Daschat installation
            data (dict): Additional data
        """
        self.config: Config = config
        self.settings: BaseSettings = settings
        self.data: dict = data
        self.init_kwargs = kwargs
        self.media_bucket = f"s3://{settings.BUCKET_FS}"

    @abstractmethod
    async def info(self, full_info=True) -> bool:
        """Get info about handsoff app

        Args:
            full_info (bool, optional): Get full info or only test connection. Defaults to True.

        Raises:
            NotImplementedError: When not implemented

        Returns:
            bool: True if can get info, False otherwise
        """

    @abstractmethod
    async def webhook(self, data_in: dict, **kwargs) -> bool:
        """
        Abstract method to process webhook calls from handoff application"""
        pass

    @abstractmethod
    async def send_msg(
        self, contact: Contact, chat: Chat, messages: List[MessageSchema], **kwargs
    ) -> bool:
        """
        Abstract method to send messages coming from contact to handoff application"""
        pass

    @abstractmethod
    async def send_operator_msg(
        self, contact: Contact, chat: Chat, messages: List[MessageSchema], **kwargs
    ) -> bool:
        """
        Abstract method to send messages coming from Daschat yourself or
        any of it plugins/bots to handoff application chat agent"""
        pass

    async def _get_webhook_type(self, type_map: dict, value: str) -> bool:
        """_get_webhook_type from data received from handoff app

        Parameters
        ----------
        type_map : dict
            Map of value to Daschat WebhookTypesEnum
        value : str
            Value received from handoff app identifying the webhook type

        Returns
        -------
        bool
            True if can associate the value with any item of WebhookTypesEnum
        """
        self._webhook_type = type_map.get(value, 0)
        if self._webhook_type == 0:
            raise NotImplementedError(f"Unknow webhook type: {value}")
        return True

    async def take_action(self, method: str, step: str = "1", **kwargs):
        """Method to take action based in some variables"""
        state: dict = {
            "plugin": self.plugin,
            "module": self.__class__.__module__,
            "class": self.__class__.__name__,
            "assignment": self._assignment,
            "method": method,
            "to_type": "owner",
            "to": "",
            "step": step,
            "config_key": self.config.key,
            "status_code": kwargs.pop("status_code", 0),
            "status": kwargs.pop("status", "STATUS-UNKNOW"),
            "errors": [],
            "kwargs": {},
        }
        if "e" in kwargs:
            state["errors"].append(str(kwargs.pop("e")))
        state["kwargs"] = kwargs
        if method.upper() == "LOGIN":
            # Probably configuration error, notify the owner
            if len(state["errors"]) == 0:
                logger.warning(json.dumps(state))
            # Probably programming error, notify DevOps
            else:
                state["to_type"] = ActionToTypeEnum.devops
                logger.error(json.dumps(state))

            # Save action
            self._actions.append(state)
        else:
            # Probably configuration error, notify the owner
            if len(state["errors"]) == 0:
                logger.warning(json.dumps(state))
            # Probably programming error, notify DevOps
            else:
                state["to_type"] = ActionToTypeEnum.devops
                logger.error(json.dumps(state))

            # Save action
            self._actions.append(state)


# end class HandoffBase



""" DasChat type definitions

https://stackoverflow.com/questions/24487405/enum-getting-value-of-enum-on-string-conversion
"""
from enum import Enum
from typing import Any, Dict, List, Optional


class MessageTypeEnum(str, Enum):
    text = "text"
    image = "image"
    audio = "audio"
    video = "video"
    file = "file"
    location = "location"
    contact = "contact"
    template = "template"
    sticker = "sticker"

    # def __repr__(self):
    #     return self.value


class MessageDispatchEnum(str, Enum):
    """MessageDispatchEnum

    Types that define which module of the DasChat ecosystem will process the message.

    See:

    https://medium.com/ingeniouslysimple/static-and-dynamic-dispatch-324d3dc890a3
    """

    gateway = "gateway"
    # action = "action"
    # flow = "flow"
    # storage = "storage"
    # contact = "contact"
    # bot = "bot"
    # plugin = "plugin"
    # webhook = "webhook"
    # system = "system"

    # def __repr__(self):
    #     return self.value


class ActionToTypeEnum(str, Enum):
    """ActionToTypeEnum

    Actions types.
    """

    contact = "contact"
    owner = "owner"
    bot = "bot"
    plugin = "plugin"
    devops = "devops"
    webhook = "webhook"
    system = "system"

    # def __repr__(self):
    #     return self.value



# type: ignore[attr-defined]
"""Daschat base fro modules and plugins."""

from pkg_resources import DistributionNotFound, get_distribution

from daschat_base.version import VERSION

try:
    __version__ = VERSION  # get_distribution("daschat_base").version
except DistributionNotFound:
    __version__ = "(local)"

__author__ = "Daschat.io"
__email__ = "admin@daschat.io"

try:
    import cython  # type: ignore
except ImportError:
    compiled: bool = False
else:  # pragma: no cover
    try:
        compiled = cython.compiled
    except AttributeError:
        compiled = False

# __all__ = "__version__", "__author__", "__email__", "compiled"



""" DasChat standard result types


"""
from __future__ import annotations

from typing import Any, List, Optional

from pydantic import BaseModel, Extra, Field
from typing_extensions import Annotated

from .schemas import DispatchCallOutSchema, ResultFieldSchema


class Param(BaseModel):
    class Config:
        allow_population_by_field_name = True
        allow_mutation = False
        extra = Extra.forbid

    name: str
    type: Any
    min_size: Optional[int] = 1
    max_size: Optional[int] = 64


class Result(BaseModel):
    class Config:
        allow_population_by_field_name = True
        allow_mutation = False
        extra = Extra.forbid

    id: str
    status: bool
    params: Optional[
        List[Annotated[Param, Field(title="Params", description="Params List")]]
    ] = []


class SystemDispatchMessages(BaseModel):
    class Config:
        allow_population_by_field_name = True
        allow_mutation = False
        extra = Extra.forbid

    # Low level messages
    info: Optional[Annotated[Result, Field(alias="INFO")]]
    warning: Optional[Annotated[Result, Field(alias="WARNING")]]
    error: Optional[Annotated[Result, Field(alias="ERROR")]]
    critical: Optional[Annotated[Result, Field(alias="CRITICAL")]]

    # Daschat messages
    success: Optional[Annotated[Result, Field(alias="SUCCESS")]]
    no_agent_online: Optional[Annotated[Result, Field(alias="NO_AGENT_ONLINE")]]
    unable_to_create_contact: Optional[
        Annotated[Result, Field(alias="UNABLE_TO_CREATE_CONTACT")]
    ]
    not_logged_in: Optional[Annotated[Result, Field(alias="NOT_LOGGED_IN")]]
    logged_in: Optional[Annotated[Result, Field(alias="LOGGED_IN")]]


MSGS = SystemDispatchMessages(
    info=Result(
        id="INFO",
        status=True,
        params=[{"name": "message", "type": str, "max_size": 1024}],
    ),
    warning=Result(
        id="WARNING",
        status=True,
        params=[{"name": "message", "type": str, "max_size": 1024}],
    ),
    error=Result(
        id="ERROR",
        status=False,
        params=[{"name": "message", "type": str, "max_size": 1024}],
    ),
    critical=Result(
        id="CRITICAL",
        status=False,
        params=[{"name": "message", "type": str, "max_size": 1024}],
    ),
    success=Result(id="SUCCESS", status=True),
    no_agent_online=Result(id="NO_AGENT_ONLINE", status=False),
    not_logged_in=Result(
        id="NOT_LOGGED_IN",
        status=False,
        params=[{"name": "user", "type": str, "min_size": 1, "max_size": 256}],
    ),
    logged_in=Result(
        id="LOGGED_IN",
        status=True,
    ),
    unable_to_create_contact=Result(
        id="UNABLE_TO_CREATE_CONTACT",
        status=False,
        params=[{"name": "contact", "type": str, "min_size": 1, "max_size": 256}],
    ),
)


def result_factory(msg: Result, **kwargs) -> ResultFieldSchema:
    """result_factory Generate result as expected by Daschat

    Examples:
        from daschat_base.messages import MSGS, msg_factory
        msg_factory(MSGS.success)
        msg_factory(MSGS.not_logged_in, user="abner")

    Args:
        msg (Result): Any result type in the MSGS contant

    Raises:
        ValueError: Parameter name not allowed
        ValueError: Result don't accept params
        ValueError: Wrong number of params
        ValueError: Wrong parameter type
        ValueError: Wrong parameter size

    Returns:
        ResultFieldSchema: [description]
    """
    call_params: int = len(kwargs)
    msg_params: int = len(msg.params)
    params: dict = {}

    if call_params > 0 and msg_params == 0:
        raise ValueError("This message do not accept params")
    if not call_params == msg_params:
        raise ValueError(
            f"Wrong number of params. This message only accepts {msg_params} parameter(s)"
        )
    if len(kwargs) > 0:
        for k in kwargs:
            param_def = next((item for item in msg.params if item.name == k), None)
            if param_def is None:
                raise ValueError(f"This parameter name is not allowed: {k}")
            if not type(kwargs[k]) == param_def.type:
                raise ValueError(
                    f"Wrong parameter type: '{k}' must be {param_def.type}"
                )
            if param_def.type == str:
                if not param_def.min_size <= len(kwargs[k]) <= param_def.max_size:
                    raise ValueError(
                        f"Wrong parameter size: '{k}' must be between {param_def.min_size} and {param_def.max_size}"
                    )
            params[k] = kwargs[k]

    return ResultFieldSchema(id=msg.id, status=msg.status, params=params)


def dispatch_factory(msg: Result, **kwargs) -> DispatchCallOutSchema:
    """result_factory Generate result as expected by Daschat

    Examples:
        from daschat_base.messages import MSGS, msg_factory
        msg_factory(MSGS.success)
        msg_factory(MSGS.not_logged_in, user="abner")

    Args:
        msg (Result): Any result type in the MSGS contant

    Raises:
        ValueError: Parameter name not allowed
        ValueError: Result don't accept params
        ValueError: Wrong number of params
        ValueError: Wrong parameter type
        ValueError: Wrong parameter size

    Returns:
        ResultFieldSchema: [description]
    """
    call_params: int = len(kwargs)
    msg_params: int = len(msg.params)
    params: dict = {}

    if call_params > 0 and msg_params == 0:
        raise ValueError("This message do not accept params")
    if not call_params == msg_params:
        raise ValueError(
            f"Wrong number of params. This message only accepts {msg_params} parameter(s)"
        )
    if len(kwargs) > 0:
        for k in kwargs:
            param_def = next((item for item in msg.params if item.name == k), None)
            if param_def is None:
                raise ValueError(f"This parameter name is not allowed: {k}")
            if not type(kwargs[k]) == param_def.type:
                raise ValueError(
                    f"Wrong parameter type: '{k}' must be {param_def.type}"
                )
            if param_def.type == str:
                if not param_def.min_size <= len(kwargs[k]) <= param_def.max_size:
                    raise ValueError(
                        f"Wrong parameter size: '{k}' must be between {param_def.min_size} and {param_def.max_size}"
                    )
            params[k] = kwargs[k]

    return DispatchCallOutSchema(
        result=ResultFieldSchema(id=msg.id, status=msg.status, params=params)
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['daschat_base']

package_data = \
{'': ['*']}

install_requires = \
['cuid>=0.3,<0.4', 'loguru>=0.5.3,<0.6.0', 'pydantic>=1.8.2,<2.0.0']

setup_kwargs = {
    'name': 'daschat-base',
    'version': '0.14.0',
    'description': 'Base package for Daschat modules.',
    'long_description': '# daschat-handsoff-base\n\nBase package for the development of integration modules with chat applications for the handsoff of conversations.\n\n\n<p align="center">\n<a href="https://www.python.org/">\n    <img src="https://img.shields.io/badge/Made%20with-Python-1f425f.svg"\n        alt = "Made with Python">\n</a>\n\n<a href="https://pypi.python.org/pypi/daschat_base">\n    <img src="https://img.shields.io/pypi/v/daschat_base.svg"\n        alt = "Release Status">\n</a>\n\n<a href="https://github.com/daschat-io/daschat_base/actions">\n    <img src="https://github.com/daschat-io/daschat_base/actions/workflows/main.yml/badge.svg?branch=release" alt="CI Status">\n</a>\n\n<!-- <a href="https://daschat-handsoff-base.readthedocs.io/en/latest/?badge=latest">\n    <img src="https://readthedocs.org/projects/daschat-base/badge/?version=latest" alt="Documentation Status">\n</a> -->\n\n</p>\n\n## Features\n\n* Free software: MIT\n* Documentation: <https://daschat-io.github.io/daschat_base/>\n\n\n## Enabling Python versions\n\nInstall Python versions using `pyenv` and enable all versions to be used with `tox`:\n\n``` console\n$ pyenv install 3.8.10 3.9.5\n$ pyenv shell 3.8.10 3.9.5\n$ pyenv local 3.8.10\n```\n\n## Acknowledgements\n\n - [Cookiecutter](https://github.com/audreyr/cookiecutter)\n - [zillionare/cookiecutter-pypackage](https://github.com/zillionare/cookiecutter-pypackage)\n - [Naereen/badges](https://github.com/Naereen/badges)\n',
    'author': 'Abner G Jacobsen',
    'author_email': 'abner@apoana.com.br',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/daschat-io/daschat_base',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



VERSION = "0.14.0"


def version_info() -> str:
    import platform
    import sys
    from importlib import import_module
    from pathlib import Path

    from daschat_base import compiled

    optional_deps = []
    for p in ("pydantic", "cuid", "loguru", "fastapi"):
        try:
            import_module(p.replace("-", "_"))
        except ImportError:
            continue
        optional_deps.append(p)

    info = {
        "daschat-base version": VERSION,
        "daschat-base compiled": compiled,
        "install path": Path(__file__).resolve().parent,
        "python version": sys.version,
        "platform": platform.platform(),
        "deps. installed": optional_deps,
    }
    return "\n".join(
        "{:>30} {}".format(k + ":", str(v).replace("\n", " ")) for k, v in info.items()
    )
